//
//  STPAddCardViewControllerTest.m
//  Stripe
//
//  Created by Ben Guo on 7/5/16.
//  Copyright © 2016 Stripe, Inc. All rights reserved.
//

#import <XCTest/XCTest.h>
#import <OCMock/OCMock.h>
#import <Stripe/Stripe.h>
#import "NSError+Stripe.h"
#import "NSLocale+STPSwizzling.h"
#import "STPCard.h"
#import "STPFixtures.h"
#import "STPPaymentCardTextFieldCell.h"
#import "STPPostalCodeValidator.h"

@interface STPAddCardViewController (Testing)
@property (nonatomic) STPPaymentCardTextFieldCell *paymentCell;
@property (nonatomic) STPAPIClient *apiClient;
@property (nonatomic) BOOL loading;
@end

@interface STPToken (Testing)
@property (nonatomic, nonnull) NSString *tokenId;
@end

@interface STPAddCardViewControllerTest : XCTestCase
@end

@implementation STPAddCardViewControllerTest

- (STPAddCardViewController *)buildAddCardViewController {
    STPPaymentConfiguration *config = [STPFixtures paymentConfiguration];
    STPTheme *theme = [STPTheme defaultTheme];
    STPAddCardViewController *vc = [[STPAddCardViewController alloc] initWithConfiguration:config
                                                                                     theme:theme];
    XCTAssertNotNil(vc.view);
    return vc;
}

- (void)testPrefilledBillingAddress_removeAddress {
    STPPaymentConfiguration *config = [STPFixtures paymentConfiguration];
    config.requiredBillingAddressFields = STPBillingAddressFieldsZip;
    STPAddCardViewController *sut = [[STPAddCardViewController alloc] initWithConfiguration:config
                                                                                      theme:[STPTheme defaultTheme]];
    STPAddress *address = [STPAddress new];
    address.name = @"John Smith Doe";
    address.phone = @"8885551212";
    address.email = @"foo@example.com";
    address.line1 = @"55 John St";
    address.city = @"Harare";
    address.postalCode = @"10002";
    address.country = @"ZW"; // Zimbabwe does not require zip codes, while the default locale for tests (US) does
    // Sanity checks
    XCTAssertFalse([STPPostalCodeValidator postalCodeIsRequiredForCountryCode:@"ZW"]);
    XCTAssertTrue([STPPostalCodeValidator postalCodeIsRequiredForCountryCode:@"US"]);

    STPUserInformation *prefilledInfo = [[STPUserInformation alloc] init];
    prefilledInfo.billingAddress = address;
    sut.prefilledInformation = prefilledInfo;

    XCTAssertNoThrow([sut loadView]);
    XCTAssertNoThrow([sut viewDidLoad]);
}

- (void)testPrefilledBillingAddress_addAddress {
    [NSLocale stp_setCurrentLocale:[NSLocale localeWithLocaleIdentifier:@"en_ZW"]]; // Zimbabwe does not require zip codes, while the default locale for tests (US) does
    // Sanity checks
    XCTAssertFalse([STPPostalCodeValidator postalCodeIsRequiredForCountryCode:@"ZW"]);
    XCTAssertTrue([STPPostalCodeValidator postalCodeIsRequiredForCountryCode:@"US"]);
    STPPaymentConfiguration *config = [STPFixtures paymentConfiguration];
    config.requiredBillingAddressFields = STPBillingAddressFieldsZip;
    STPAddCardViewController *sut = [[STPAddCardViewController alloc] initWithConfiguration:config
                                                                                      theme:[STPTheme defaultTheme]];
    STPAddress *address = [STPAddress new];
    address.name = @"John Smith Doe";
    address.phone = @"8885551212";
    address.email = @"foo@example.com";
    address.line1 = @"55 John St";
    address.city = @"New York";
    address.state = @"NY";
    address.postalCode = @"10002";
    address.country = @"US";

    STPUserInformation *prefilledInfo = [[STPUserInformation alloc] init];
    prefilledInfo.billingAddress = address;
    sut.prefilledInformation = prefilledInfo;

    XCTAssertNoThrow([sut loadView]);
    XCTAssertNoThrow([sut viewDidLoad]);
    [NSLocale stp_resetCurrentLocale];
}


#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"

- (void)testNextWithCreateTokenError {
    STPAddCardViewController *sut = [self buildAddCardViewController];
    STPCardParams *expectedCardParams = [STPFixtures cardParams];
    sut.paymentCell.paymentField.cardParams = expectedCardParams;

    id mockAPIClient = OCMClassMock([STPAPIClient class]);
    sut.apiClient = mockAPIClient;
    XCTestExpectation *exp = [self expectationWithDescription:@"createTokenWithCard"];
    OCMStub([mockAPIClient createTokenWithCard:[OCMArg any] completion:[OCMArg any]])
    .andDo(^(NSInvocation *invocation){
        STPCardParams *cardParams;
        STPTokenCompletionBlock completion;
        [invocation getArgument:&cardParams atIndex:2];
        [invocation getArgument:&completion atIndex:3];
        XCTAssertEqualObjects(cardParams.number, expectedCardParams.number);
        XCTAssertTrue(sut.loading);
        NSError *error = [NSError stp_genericFailedToParseResponseError];
        completion(nil, error);
        XCTAssertFalse(sut.loading);
        [exp fulfill];
    });

    // tap next button
    UIBarButtonItem *nextButton = sut.navigationItem.rightBarButtonItem;
    [nextButton.target performSelector:nextButton.action withObject:nextButton];

    [self waitForExpectationsWithTimeout:2 handler:nil];
}

- (void)testNextWithCreateTokenSuccessAndDidCreateTokenError {
    STPAddCardViewController *sut = [self buildAddCardViewController];

    id mockAPIClient = OCMClassMock([STPAPIClient class]);
    id mockDelegate = OCMProtocolMock(@protocol(STPAddCardViewControllerDelegate));
    sut.apiClient = mockAPIClient;
    sut.delegate = mockDelegate;
    STPCardParams *expectedCardParams = [STPFixtures cardParams];
    sut.paymentCell.paymentField.cardParams = expectedCardParams;

    STPToken *expectedToken = [STPToken new];
    expectedToken.tokenId = @"tok_123";
    XCTestExpectation *createTokenExp = [self expectationWithDescription:@"createTokenWithCard"];
    OCMStub([mockAPIClient createTokenWithCard:[OCMArg any] completion:[OCMArg any]])
    .andDo(^(NSInvocation *invocation){
        STPCardParams *cardParams;
        STPTokenCompletionBlock completion;
        [invocation getArgument:&cardParams atIndex:2];
        [invocation getArgument:&completion atIndex:3];
        XCTAssertEqualObjects(cardParams.number, expectedCardParams.number);
        XCTAssertTrue(sut.loading);
        completion(expectedToken, nil);
        [createTokenExp fulfill];
    });
    
    XCTestExpectation *didCreateTokenExp = [self expectationWithDescription:@"didCreateToken"];
    OCMStub([mockDelegate addCardViewController:[OCMArg any] didCreateToken:[OCMArg any] completion:[OCMArg any]])
    .andDo(^(NSInvocation *invocation){
        STPToken *token;
        STPErrorBlock completion;
        [invocation getArgument:&token atIndex:3];
        [invocation getArgument:&completion atIndex:4];
        XCTAssertTrue(sut.loading);
        NSError *error = [NSError stp_genericFailedToParseResponseError];
        XCTAssertEqualObjects(token.tokenId, expectedToken.tokenId);
        completion(error);
        XCTAssertFalse(sut.loading);
        [didCreateTokenExp fulfill];
    });

    // tap next button
    UIBarButtonItem *nextButton = sut.navigationItem.rightBarButtonItem;
    [nextButton.target performSelector:nextButton.action withObject:nextButton];

    [self waitForExpectationsWithTimeout:2 handler:nil];
}

- (void)testNextWithCreateTokenSuccessAndDidCreateTokenSuccess {
    STPAddCardViewController *sut = [self buildAddCardViewController];

    id mockAPIClient = OCMClassMock([STPAPIClient class]);
    id mockDelegate = OCMProtocolMock(@protocol(STPAddCardViewControllerDelegate));
    sut.apiClient = mockAPIClient;
    sut.delegate = mockDelegate;
    STPCardParams *expectedCardParams = [STPFixtures cardParams];
    sut.paymentCell.paymentField.cardParams = expectedCardParams;

    STPToken *expectedToken = [STPToken new];
    expectedToken.tokenId = @"tok_123";
    XCTestExpectation *createTokenExp = [self expectationWithDescription:@"createTokenWithCard"];
    OCMStub([mockAPIClient createTokenWithCard:[OCMArg any] completion:[OCMArg any]])
    .andDo(^(NSInvocation *invocation){
        STPCardParams *cardParams;
        STPTokenCompletionBlock completion;
        [invocation getArgument:&cardParams atIndex:2];
        [invocation getArgument:&completion atIndex:3];
        XCTAssertEqualObjects(cardParams.number, expectedCardParams.number);
        XCTAssertTrue(sut.loading);
        completion(expectedToken, nil);
        [createTokenExp fulfill];
    });

    XCTestExpectation *didCreateTokenExp = [self expectationWithDescription:@"didCreateToken"];
    OCMStub([mockDelegate addCardViewController:[OCMArg any] didCreateToken:[OCMArg any] completion:[OCMArg any]])
    .andDo(^(NSInvocation *invocation){
        STPToken *token;
        STPErrorBlock completion;
        [invocation getArgument:&token atIndex:3];
        [invocation getArgument:&completion atIndex:4];
        XCTAssertTrue(sut.loading);
        XCTAssertEqualObjects(token.tokenId, expectedToken.tokenId);
        completion(nil);
        XCTAssertFalse(sut.loading);
        [didCreateTokenExp fulfill];
    });

    // tap next button
    UIBarButtonItem *nextButton = sut.navigationItem.rightBarButtonItem;
    [nextButton.target performSelector:nextButton.action withObject:nextButton];

    [self waitForExpectationsWithTimeout:2 handler:nil];
}

/**
 Tests that setting createCardSource creates a card source instead of a card
 token and calls the correct delegate method.
 */
- (void)testCreatesCardSource {
    STPPaymentConfiguration *config = [STPFixtures paymentConfiguration];
    config.createCardSources = YES;
    STPTheme *theme = [STPTheme defaultTheme];
    STPAddCardViewController *sut = [[STPAddCardViewController alloc] initWithConfiguration:config
                                                                                     theme:theme];
    XCTAssertNotNil(sut.view);

    id mockAPIClient = OCMClassMock([STPAPIClient class]);
    id mockDelegate = OCMProtocolMock(@protocol(STPAddCardViewControllerDelegate));
    sut.apiClient = mockAPIClient;
    sut.delegate = mockDelegate;
    STPCardParams *expectedCardParams = [STPFixtures cardParams];
    sut.paymentCell.paymentField.cardParams = expectedCardParams;

    STPSource *expectedSource = [STPFixtures cardSource];
    XCTestExpectation *createSourceExp = [self expectationWithDescription:@"createSource"];
    OCMStub([mockAPIClient createSourceWithParams:[OCMArg any] completion:[OCMArg any]])
    .andDo(^(NSInvocation *invocation){
        STPSourceParams *sourceParams;
        STPSourceCompletionBlock completion;
        [invocation getArgument:&sourceParams atIndex:2];
        [invocation getArgument:&completion atIndex:3];
        XCTAssertEqualObjects(sourceParams.additionalAPIParameters[@"card"][@"number"], expectedCardParams.number);
        XCTAssertTrue(sut.loading);
        completion(expectedSource, nil);
        [createSourceExp fulfill];
    });

    XCTestExpectation *didCreateSourceExp = [self expectationWithDescription:@"didCreateSource"];
    OCMStub([mockDelegate addCardViewController:[OCMArg any] didCreateSource:[OCMArg any] completion:[OCMArg any]])
    .andDo(^(NSInvocation *invocation){
        STPSource *source;
        STPErrorBlock completion;
        [invocation getArgument:&source atIndex:3];
        [invocation getArgument:&completion atIndex:4];
        XCTAssertTrue(sut.loading);
        XCTAssertEqualObjects(source.stripeID, expectedSource.stripeID);
        completion(nil);
        XCTAssertFalse(sut.loading);
        [didCreateSourceExp fulfill];
    });

    // tap next button
    UIBarButtonItem *nextButton = sut.navigationItem.rightBarButtonItem;
    [nextButton.target performSelector:nextButton.action withObject:nextButton];

    [self waitForExpectationsWithTimeout:2 handler:nil];
}

#pragma clang diagnostic pop

@end
